// 4D Systems

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <pthread.h>
#include <math.h>


#include <geniePi.h>  //the ViSi-Genie-RaspPi library

#define false 0
#define true  1

// If not using Flashbank0 for Update and Run Flashbank, use this:
#define BANKS 2    // Number of Banks used
#define PROJECT0 0 // Bank used for ViSi-Genie Project "BnkPi0"
#define PROJECT1 1 // Bank used for ViSi-Genie Project "BnkPi1"

// Otherwise, use this:
// #define BANKS 3    // Number of Banks used
// #define PROJECT0 1 // Bank used for ViSi-Genie Project "BnkPi0"
// #define PROJECT1 2 // Bank used for ViSi-Genie Project "BnkPi1"

#define GENIE_OBJ_MAGIC 34

int currentBank = PROJECT0; // initialize to Project0
int changingBanks = true;
int switchBankFlag = false;
int sliderVal = 0;
int smartSliderVal = 0;

int gaugeVal[BANKS] = {0, 0};   //holds the value of the gauges
int step[BANKS] = {1};       //increment or decrement value, initialized to 1.

void writePrevValues() {
  if (currentBank == PROJECT0) {
    genieWriteObj(GENIE_OBJ_SLIDER, 0x00, sliderVal);
    genieWriteObj(GENIE_OBJ_LED_DIGITS, 0x00, sliderVal);
    genieWriteObj(GENIE_OBJ_COOL_GAUGE, 0x00, gaugeVal[currentBank]);
  } else if (currentBank == PROJECT1) {
    genieWriteObj(GENIE_OBJ_ISMARTSLIDER, 0x00, smartSliderVal);
    genieWriteObj(GENIE_OBJ_LED_DIGITS, 0x00, smartSliderVal);
    genieWriteObj(GENIE_OBJ_ISMARTGAUGE, 0x00, gaugeVal[currentBank]);
  }
  char temp[10] = "";
  sprintf(temp, "Bank %d!!", currentBank);
  genieWriteStr(0x00, temp); //write to Strings0
}

//This a thread for writing to the gauges. The value of
//the gauges will change from 0 to 99 and then from 99 to 0.
//This process repeats forever, running in parallel with the main.
static void *handleGauge(void *data)
{

  for (;;)            //infinite loop
  {
if (!changingBanks) {
  if (currentBank == PROJECT0) {
    //write to Coolgauge0
    genieWriteObj(GENIE_OBJ_COOL_GAUGE, 0x00, gaugeVal[0]);
  } else if (currentBank == PROJECT1) {
    //write to SmartGauge0
    genieWriteObj(GENIE_OBJ_ISMARTGAUGE, 0x00, gaugeVal[1]);
  }
  //usleep(10000);    //10-millisecond delay
  gaugeVal[currentBank] += step[currentBank]; //increment or decrement
  genieWriteObj(GENIE_OBJ_USER_LED, 0x00, gaugeVal[currentBank] % 2);

  if (gaugeVal[currentBank] == 99)  step[currentBank] = -1;
  if (gaugeVal[currentBank] == 0) step[currentBank] = 1;
  usleep(10000); //10ms delay
}
    else {
      int i;
      for (i = 0; changingBanks ; i++) {
	printf(".");
        usleep(10000);
        if (changingBanks && (i % 50 == 0))
        {
          currentBank = genieReadObj(GENIE_OBJ_MAGIC, 0x00);
          if (currentBank != -1) // Query current bank if post genie report event current bank wasn't receive after a second);
          {
            changingBanks = false;
            writePrevValues();
          }
        }
      }
    }
  }
  return NULL;
}


//This is the event handler. Messages received from the display
//are processed here.
void handleGenieEvent(struct genieReplyStruct * reply)
{
  if (reply->cmd == GENIE_REPORT_EVENT)   //check if the cmd byte is a report event
  {
    if (reply->object == GENIE_OBJ_SLIDER) //check if the object byte is that of a slider
    {
      if (reply->index == 0)    //check if the index byte is that of Slider0
      { // write to the LED digits object
        sliderVal = reply->data;
        genieWriteObj(GENIE_OBJ_LED_DIGITS, 0x00, sliderVal);
      }
    }
    else if (reply->object == GENIE_OBJ_ISMARTSLIDER)
    {
      if (reply->index == 0)    //check if the index byte is that of SmartSlider0
      { // write to the LED digits object
        smartSliderVal = reply->data;
        genieWriteObj(GENIE_OBJ_LED_DIGITS, 0x00, smartSliderVal);
      }
    }
    else if (reply->object == GENIE_OBJ_WINBUTTON)
    {
      if (reply->index == 0) {    //check if the index byte is that of Winbutton0
        changingBanks = true;
        printf("\nCurrent Bank: Flashbank%d\n", currentBank);
        currentBank = (currentBank == PROJECT0) ? PROJECT1 : PROJECT0;
        printf("Switching to FlashBank%d\n", currentBank);
        genieWriteObj(GENIE_OBJ_MAGIC, 0, currentBank);
      }
    }
    else if (reply->object == GENIE_OBJ_MAGIC) {
      if (reply->index == 0) {
        changingBanks = false;
        currentBank = reply->data;
        writePrevValues();
      }
    }
  }

  //if the received message is not a report event, print a message on the terminal window
  else
    printf("Unhandled event: command: %2d, object: %2d, index: %d, data: %d \r\n", reply->cmd, reply->object, reply->index, reply->data);
}

int main()
{
  pthread_t myThread;              //declare a thread for gauges

  struct genieReplyStruct reply ;  //declare a genieReplyStruct type structure

  //print some information on the terminal window
  printf("\n\n");
  printf("Visi-Genie-Raspberry-Pi Banks Demo\n");
  printf("==================================\n");
  printf("Program is running. Press Ctrl + C to close.\n");

  //open the Raspberry Pi's onboard serial port, baud rate is 115200
  //make sure that the display module has the same baud rate
  genieSetup("/dev/serial0", 115200);

  //start the thread for writing to the gauge
  (void)pthread_create (&myThread,  NULL, handleGauge, NULL);

  for (;;)                        //infinite loop
  {
    while (genieReplyAvail())     //check if a message is available
    {
      genieGetReply(&reply);      //take out a message from the events buffer
      handleGenieEvent(&reply);   //call the event handler to process the message
    }
    usleep(10000);                //10-millisecond delay.Don't hog the
  }                           //CPU in case anything else is happening
  return 0;
}


